from featuretools.primitives import AggregationPrimitive
from tsfresh.feature_extraction.settings import ComprehensiveFCParameters

from featuretools_tsfresh_primitives.primitives import SUPPORTED_PRIMITIVES

PRIMITIVES = {primitive.name: primitive for primitive in SUPPORTED_PRIMITIVES}


def comprehensive_fc_parameters():
    """A wrapper around the tsfresh function :class:`ComperehensiveFCParameters` to filter out unsupported parameter settings.

    Returns:
        parameters (dict) : a dictionary list of parameters

    Docstring source:
    https://tsfresh.readthedocs.io/en/latest/api/tsfresh.feature_extraction.html#tsfresh.feature_extraction.settings.ComprehensiveFCParameters
    """
    parameters = ComprehensiveFCParameters()

    # when a partial autocorrelation has a lag of zero,
    # an error is raised from `tsfresh.feature_extraction.extract_features`
    partial_autocorrelation = parameters['partial_autocorrelation']
    for index, values in enumerate(partial_autocorrelation):
        if values['lag'] == 0: del partial_autocorrelation[index]

    return parameters


def primitives_from_fc_settings(fc_settings=None):
    """Returns a list of :class:`AggregationPrimitive` from tsfresh settings.
    The format is the same as the argument `default_fc_parameters` of :func:`tsfresh.feature_extraction.extract_features`

    Args:
        fc_settings (dict): mapping from tsfresh feature calculator names (snake case) to parameters.
            Only those names which are keys in this dict will be calculated.
            The default value is the dictionary of parameter settings from :class:`ComperehensiveFCParameters`.

    Returns:
        agg_primitives (list): A list of primitive instances.

    Examples:
        >>> parameters = {'autocorrelation': [{'lag': 2}]}
        >>> primitive = primitives_from_fc_settings(parameters)[0]
        >>> primitive(range(3))
        -1.5

    Docstring source:
    https://tsfresh.readthedocs.io/en/latest/api/tsfresh.feature_extraction.html#module-tsfresh.feature_extraction.settings
    """
    parameters = fc_settings or comprehensive_fc_parameters()
    agg_primitives = []

    def add_primitive_instances(primitive, primitives):
        inputs = parameters[primitive.name] or [{}]

        for values in inputs:
            instance = primitive(**values)
            primitives.append(instance)

    for key in parameters:
        if key in PRIMITIVES:
            primitive = PRIMITIVES[key]
            assert issubclass(primitive, AggregationPrimitive)
            add_primitive_instances(primitive, agg_primitives)

    return agg_primitives



from featuretools.primitives import AggregationPrimitive
from featuretools.variable_types import Boolean, Numeric
from tsfresh.feature_extraction.feature_calculators import symmetry_looking


class SymmetryLooking(AggregationPrimitive):
    """Boolean variable denoting if the distribution looks symmetric.

    Args:
        r (float) : Percentage of the range to compare with.

    Docstring source:
    https://tsfresh.readthedocs.io/en/latest/api/tsfresh.feature_extraction.html#tsfresh.feature_extraction.feature_calculators.symmetry_looking
    """
    name = "symmetry_looking"
    input_types = [Numeric]
    return_type = Boolean
    stack_on_self = False

    def __init__(self, r):
        self.r = r

    def get_function(self):
        def function(x):
            param = [{'r': self.r}]
            return symmetry_looking(x, param)[0][1]

        return function



# -*- coding: utf-8 -*-

from featuretools.primitives import AggregationPrimitive
from featuretools.variable_types import Numeric
from tsfresh.feature_extraction.feature_calculators import \
    time_reversal_asymmetry_statistic


class TimeReversalAsymmetryStatistic(AggregationPrimitive):
    """This function calculates the value of

    .. math::

        \\frac{1}{n-2lag} \\sum_{i=0}^{n-2lag} x_{i + 2 \\cdot lag}^2 \\cdot x_{i + lag} - x_{i + lag} \\cdot  x_{i}^2

    which is

    .. math::

        \\mathbb{E}[L^2(X)^2 \\cdot L(X) - L(X) \\cdot X^2]

    where :math:`\\mathbb{E}` is the mean and :math:`L` is the lag operator.
    It was proposed in [1] as a promising feature to extract from time series.

    .. rubric:: References

    |  [1] Fulcher, B.D., Jones, N.S. (2014).
    |  Highly comparative feature-based time-series classification.
    |  Knowledge and Data Engineering, IEEE Transactions on 26, 3026–3037.

    Args:
        lag (int) : The lag that should be used in the calculation of the feature.

    Docstring source:
    https://tsfresh.readthedocs.io/en/latest/api/tsfresh.feature_extraction.html#tsfresh.feature_extraction.feature_calculators.time_reversal_asymmetry_statistic
    """
    name = "time_reversal_asymmetry_statistic"
    input_types = [Numeric]
    return_type = Numeric
    stack_on_self = False

    def __init__(self, lag):
        self.lag = lag

    def get_function(self):
        def function(x):
            return time_reversal_asymmetry_statistic(x, lag=self.lag)

        return function



from featuretools.primitives import AggregationPrimitive
from featuretools.variable_types import Numeric
from tsfresh.feature_extraction.feature_calculators import quantile


class Quantile(AggregationPrimitive):
    """Calculates the q quantile of x. This is the value of x greater than q%
    of the ordered values from x.

    Args:
        q (float) : The quantile to calculate.

    Docstring source:
    https://tsfresh.readthedocs.io/en/latest/api/tsfresh.feature_extraction.html#tsfresh.feature_extraction.feature_calculators.quantile
    """
    name = "quantile"
    input_types = [Numeric]
    return_type = Numeric
    stack_on_self = False

    def __init__(self, q):
        self.q = q

    def get_function(self):
        def function(x):
            return quantile(x, q=self.q)

        return function



from featuretools.primitives import AggregationPrimitive
from featuretools.variable_types import Numeric
from tsfresh.feature_extraction.feature_calculators import \
    sum_of_reoccurring_data_points


class SumOfReoccurringDataPoints(AggregationPrimitive):
    """Returns the sum of all data points, that are present in the time series
    more than once.

    Docstring source:
    https://tsfresh.readthedocs.io/en/latest/api/tsfresh.feature_extraction.html#tsfresh.feature_extraction.feature_calculators.sum_of_reoccurring_data_points
    """
    name = "sum_of_reoccurring_data_points"
    input_types = [Numeric]
    return_type = Numeric
    stack_on_self = False

    def get_function(self):
        return sum_of_reoccurring_data_points



import featuretools as ft
from numpy.testing import assert_almost_equal
from pytest import fixture, mark
from tsfresh.feature_extraction import extract_features

from featuretools_tsfresh_primitives import (SUPPORTED_PRIMITIVES,
                                             comprehensive_fc_parameters,
                                             primitives_from_fc_settings)

PARAMETERS = comprehensive_fc_parameters()


@fixture(scope='session')
def entityset():
    return ft.demo.load_mock_customer(return_entityset=True)


@fixture(scope='session')
def df(entityset):
    df = entityset['transactions'].df
    df = df.filter(regex='session_id|transaction_time|amount')
    df = df.set_index('transaction_time').sort_index()
    return df


def parametrize():
    values = {'argvalues': [], 'ids': []}
    for primitive in SUPPORTED_PRIMITIVES:
        parameter_list = PARAMETERS[primitive.name] or [{}]
        primitive_settings = {primitive.name: parameter_list}
        primitives = primitives_from_fc_settings(primitive_settings)
        items = zip(parameter_list, primitives)

        for parameters, primitive in items:
            item = {primitive.name: [parameters]}, primitive
            values['argvalues'].append(item)

            name = primitive.name.upper()
            args = primitive.get_args_string()
            if args: name += '(%s)' % args.lstrip(' ,')
            values['ids'].append(name)

    return values


@mark.parametrize('parameters,primitive', **parametrize())
def test_primitive(entityset, df, parameters, primitive):
    expected = extract_features(
        timeseries_container=df,
        column_id='session_id',
        default_fc_parameters=parameters,
    )

    base = entityset['transactions']['amount']

    if primitive.name == 'linear_trend_timewise':
        base = [base, entityset['transactions']['transaction_time']]

    feature = ft.Feature(
        base=base,
        parent_entity=entityset['sessions'],
        primitive=primitive,
    )

    actual = ft.calculate_feature_matrix(
        features=[feature],
        entityset=entityset,
    )

    assert_almost_equal(
        actual=actual.values,
        desired=expected.values,
    )



from featuretools.primitives import AggregationPrimitive
from featuretools.variable_types import Numeric
from tsfresh.feature_extraction.feature_calculators import agg_linear_trend


class AggLinearTrend(AggregationPrimitive):
    """Calculates a linear least-squares regression for values of the time
    series that were aggregated over chunks versus the sequence from 0 up to
    the number of chunks minus one.

    Args:
        attr (str) : Controls which of the characteristics are returned.
            Possible extracted attributes are: ['pvalue', 'rvalue',
            'intercept', 'slope', 'stderr'].
        chunk_len (int) : How many time series values are in each chunk.
        f_agg (str) : Name of the aggregator function. Possible values are:
            ['max', 'min' or , 'mean', 'median'].

    Docstring source:
    https://tsfresh.readthedocs.io/en/latest/api/tsfresh.feature_extraction.html#tsfresh.feature_extraction.feature_calculators.agg_linear_trend
    """
    name = "agg_linear_trend"
    input_types = [Numeric]
    return_type = Numeric
    stack_on_self = False

    def __init__(self, attr, chunk_len, f_agg):
        self.attr = attr
        self.chunk_len = chunk_len
        self.f_agg = f_agg

    def get_function(self):
        def function(x):
            param = [{'attr': self.attr, 'f_agg': self.f_agg, 'chunk_len': self.chunk_len}]
            values = list(agg_linear_trend(x.to_numpy(), param))
            return values[0][1]

        return function



from featuretools.primitives import AggregationPrimitive
from featuretools.variable_types import Boolean, Numeric
from tsfresh.feature_extraction.feature_calculators import has_duplicate


class HasDuplicate(AggregationPrimitive):
    """Checks if any value in x occurs more than once.

    Docstring source:
    https://tsfresh.readthedocs.io/en/latest/api/tsfresh.feature_extraction.html#tsfresh.feature_extraction.feature_calculators.has_duplicate
    """
    name = "has_duplicate"
    input_types = [Numeric]
    return_type = Boolean
    stack_on_self = False

    def get_function(self):
        return has_duplicate



from featuretools.primitives import AggregationPrimitive
from featuretools.variable_types import Numeric
from tsfresh.feature_extraction.feature_calculators import count_below_mean


class CountBelowMean(AggregationPrimitive):
    """Returns the number of values in x that are lower than the mean of x

    Docstring source:
    https://tsfresh.readthedocs.io/en/latest/api/tsfresh.feature_extraction.html#tsfresh.feature_extraction.feature_calculators.count_below_mean
    """
    name = "count_below_mean"
    input_types = [Numeric]
    return_type = Numeric
    stack_on_self = False

    def get_function(self):
        return count_below_mean



from featuretools.primitives import AggregationPrimitive
from featuretools.variable_types import Numeric
from tsfresh.feature_extraction.feature_calculators import change_quantiles


class ChangeQuantiles(AggregationPrimitive):
    """First fixes a corridor given by the quantiles ql and qh of the
    distribution of x. Then calculates the average, absolute value of
    consecutive changes of the series x inside this corridor.

    Args:
        ql (float) : The lower quantile of the corridor.
        qh (float) : The higher quantile of the corridor.
        isabs (bool) : Should the absolute differences be taken?
        f_agg (str) : The numpy function that aggregates (e.g. mean, var, std,
            median) the differences in the bin.

    Docstring source:
    https://tsfresh.readthedocs.io/en/latest/api/tsfresh.feature_extraction.html#tsfresh.feature_extraction.feature_calculators.change_quantiles
    """
    name = "change_quantiles"
    input_types = [Numeric]
    return_type = Numeric
    stack_on_self = False

    def __init__(self, ql, qh, isabs, f_agg):
        self.ql = ql
        self.qh = qh
        self.isabs = isabs
        self.f_agg = f_agg

    def get_function(self):
        def function(x):
            return change_quantiles(x,
                                    ql=self.ql,
                                    qh=self.qh,
                                    isabs=self.isabs,
                                    f_agg=self.f_agg)

        return function



from featuretools.primitives import AggregationPrimitive
from featuretools.variable_types import DatetimeTimeIndex, Numeric
from tsfresh.feature_extraction.feature_calculators import \
    linear_trend_timewise


class LinearTrendTimewise(AggregationPrimitive):
    """Calculate a linear least-squares regression for the values of the time series versus the sequence from 0 to length of the time series minus one.
    This feature uses the index of the time series to fit the model, which must be of a datetime dtype. The parameters control which of the characteristics are returned.
    Possible extracted attributes are "pvalue", "rvalue", "intercept", "slope", "stderr", see the documentation of linregress for more information.

    Args:
        attr (str) : Controls which of the characteristics are returned.
            Possible extracted attributes are: pvalue, rvalue, intercept, slope, or stderr.

    Docstring source:
    https://tsfresh.readthedocs.io/en/latest/api/tsfresh.feature_extraction.html#tsfresh.feature_extraction.feature_calculators.linear_trend_timewise
    """
    name = "linear_trend_timewise"
    input_types = [Numeric, DatetimeTimeIndex]
    return_type = Numeric
    stack_on_self = False

    def __init__(self, attr):
        self.attr = attr

    def get_function(self):
        def function(numeric, time):
            numeric.index = time.values
            param = [{'attr': self.attr}]
            values = linear_trend_timewise(numeric, param)
            return values[0][1]

        return function



from featuretools.primitives import AggregationPrimitive
from featuretools.variable_types import Numeric
from tsfresh.feature_extraction.feature_calculators import sample_entropy


class SampleEntropy(AggregationPrimitive):
    """Calculate and return sample entropy of x.

    .. rubric:: References

    |  [1] http://en.wikipedia.org/wiki/Sample_Entropy
    |  [2] https://www.ncbi.nlm.nih.gov/pubmed/10843903?dopt=Abstract

    Docstring source:
    https://tsfresh.readthedocs.io/en/latest/api/tsfresh.feature_extraction.html#tsfresh.feature_extraction.feature_calculators.sample_entropy
    """
    name = "sample_entropy"
    input_types = [Numeric]
    return_type = Numeric
    stack_on_self = False

    def get_function(self):
        return sample_entropy



from featuretools.primitives import AggregationPrimitive
from featuretools.variable_types import Numeric
from tsfresh.feature_extraction.feature_calculators import binned_entropy


class BinnedEntropy(AggregationPrimitive):
    """First bins the values of x into max_bins equidistant bins.
    Then calculates the value of

    .. math::

        - \\sum_{k=0}^{min(max\\_bins, len(x))} p_k log(p_k) \\cdot \\mathbf{1}_{(p_k > 0)}

    where :math:`p_k` is the percentage of samples in bin :math:`k`.

     Args:
        max_bins (int) : The maximal number of bins.

    Docstring source:
    https://tsfresh.readthedocs.io/en/latest/api/tsfresh.feature_extraction.html#tsfresh.feature_extraction.feature_calculators.binned_entropy
    """
    name = "binned_entropy"
    input_types = [Numeric]
    return_type = Numeric
    stack_on_self = False

    def __init__(self, max_bins):
        self.max_bins = max_bins

    def get_function(self):
        def function(x):
            return binned_entropy(x, max_bins=self.max_bins)

        return function



from featuretools.primitives import AggregationPrimitive
from featuretools.variable_types import Numeric
from tsfresh.feature_extraction.feature_calculators import spkt_welch_density


class SpktWelchDensity(AggregationPrimitive):
    """This feature calculator estimates the cross power spectral density of
    the time series at different frequencies. To do so, the time series is
    first shifted from the time domain to the frequency domain.

    Args:
        coeff (int) : Value of coefficient.

    Docstring source:
    https://tsfresh.readthedocs.io/en/latest/api/tsfresh.feature_extraction.html#tsfresh.feature_extraction.feature_calculators.spkt_welch_density
    """
    name = "spkt_welch_density"
    input_types = [Numeric]
    return_type = Numeric
    stack_on_self = False

    def __init__(self, coeff):
        self.coeff = coeff

    def get_function(self):
        def function(x):
            param = [{'coeff': self.coeff}]
            return list(spkt_welch_density(x, param))[0][1]

        return function




# flake8: noqa
from .abs_energy import AbsEnergy
from .absolute_sum_of_changes import AbsoluteSumOfChanges
from .agg_autocorrelation import AggAutocorrelation
from .agg_linear_trend import AggLinearTrend
from .approximate_entropy import ApproximateEntropy
from .ar_coefficient import ArCoefficient
from .augmented_dickey_fuller import AugmentedDickeyFuller
from .autocorrelation import Autocorrelation
from .binned_entropy import BinnedEntropy
from .c3 import C3
from .change_quantiles import ChangeQuantiles
from .cid_ce import CidCe
from .count_above_mean import CountAboveMean
from .count_below_mean import CountBelowMean
from .cwt_coefficients import CwtCoefficients
from .energy_ratio_by_chunks import EnergyRatioByChunks
from .fft_aggregated import FftAggregated
from .fft_coefficient import FftCoefficient
from .first_location_of_maximum import FirstLocationOfMaximum
from .first_location_of_minimum import FirstLocationOfMinimum
from .friedrich_coefficients import FriedrichCoefficients
from .has_duplicate import HasDuplicate
from .has_duplicate_max import HasDuplicateMax
from .has_duplicate_min import HasDuplicateMin
from .index_mass_quantile import IndexMassQuantile
from .kurtosis import Kurtosis
from .large_standard_deviation import LargeStandardDeviation
from .last_location_of_maximum import LastLocationOfMaximum
from .last_location_of_minimum import LastLocationOfMinimum
from .length import Length
from .linear_trend import LinearTrend
from .linear_trend_timewise import LinearTrendTimewise
from .longest_strike_above_mean import LongestStrikeAboveMean
from .longest_strike_below_mean import LongestStrikeBelowMean
from .max_langevin_fixed_point import MaxLangevinFixedPoint
from .maximum import Maximum
from .mean import Mean
from .mean_abs_change import MeanAbsChange
from .mean_change import MeanChange
from .mean_second_derivative_central import MeanSecondDerivativeCentral
from .median import Median
from .minimum import Minimum
from .number_crossing_m import NumberCrossingM
from .number_cwt_peaks import NumberCwtPeaks
from .number_peaks import NumberPeaks
from .partial_autocorrelation import PartialAutocorrelation
from .percentage_of_reoccurring_datapoints_to_all_datapoints import \
    PercentageOfReoccurringDatapointsToAllDatapoints
from .percentage_of_reoccurring_values_to_all_values import \
    PercentageOfReoccurringValuesToAllValues
from .quantile import Quantile
from .range_count import RangeCount
from .ratio_beyond_r_sigma import RatioBeyondRSigma
from .ratio_value_number_to_time_series_length import \
    RatioValueNumberToTimeSeriesLength
from .sample_entropy import SampleEntropy
from .skewness import Skewness
from .spkt_welch_density import SpktWelchDensity
from .standard_deviation import StandardDeviation
from .sum_of_reoccurring_data_points import SumOfReoccurringDataPoints
from .sum_of_reoccurring_values import SumOfReoccurringValues
from .sum_values import SumValues
from .symmetry_looking import SymmetryLooking
from .time_reversal_asymmetry_statistic import TimeReversalAsymmetryStatistic
from .value_count import ValueCount
from .variance import Variance
from .variance_larger_than_standard_deviation import \
    VarianceLargerThanStandardDeviation

SUPPORTED_PRIMITIVES = [value for value in locals().values() if isinstance(value, type)]



from featuretools.primitives import AggregationPrimitive
from featuretools.variable_types import Numeric
from tsfresh.feature_extraction.feature_calculators import \
    friedrich_coefficients


class FriedrichCoefficients(AggregationPrimitive):
    """Coefficients of polynomial :math:`h(x)`, which has been fitted to
    the deterministic dynamics of Langevin model

    .. math::
        \\dot{x}(t) = h(x(t)) + \\mathcal{N}(0,R)

    as described by [1].

    For short time-series this method is highly dependent on the parameters.

    .. rubric:: References

    |  [1] Friedrich et al. (2000): Physics Letters A 271, p. 217-222
    |  *Extracting model equations from experimental data*

    Args:
        m (int) : The order of polynom to fit for estimating fixed points of
            dynamics. Must be a positive integer.
        r (float) : The number of quantiles to use for averaging.
        coeff (int) : The coefficient. Must be a positive interger.

    Docstring source:
    https://tsfresh.readthedocs.io/en/latest/api/tsfresh.feature_extraction.html#tsfresh.feature_extraction.feature_calculators.friedrich_coefficients
    """
    name = "friedrich_coefficients"
    input_types = [Numeric]
    return_type = Numeric
    stack_on_self = False

    def __init__(self, m, r, coeff):
        self.m = m
        self.r = r
        self.coeff = coeff

    def get_function(self):
        def function(x):
            param = [{'m': self.m, 'r': self.r, 'coeff': self.coeff}]
            return list(friedrich_coefficients(x, param))[0][1]

        return function



from featuretools.primitives import AggregationPrimitive
from featuretools.variable_types import Numeric
from tsfresh.feature_extraction.feature_calculators import approximate_entropy


class ApproximateEntropy(AggregationPrimitive):
    """Implements a vectorized Approximate entropy algorithm.

    Args:
        m (int) : Length of compared run of data.
        r (float) : Filtering level, must be positive.

    Docstring source:
    https://tsfresh.readthedocs.io/en/latest/api/tsfresh.feature_extraction.html#tsfresh.feature_extraction.feature_calculators.approximate_entropy
    """
    name = "approximate_entropy"
    input_types = [Numeric]
    return_type = Numeric
    stack_on_self = False

    def __init__(self, m, r):
        self.m = m
        self.r = r

    def get_function(self):
        def function(x):
            return approximate_entropy(x, m=self.m, r=self.r)

        return function



from featuretools.primitives import AggregationPrimitive
from featuretools.variable_types import Numeric
from tsfresh.feature_extraction.feature_calculators import skewness


class Skewness(AggregationPrimitive):
    """Returns the sample skewness of x (calculated with the adjusted
    Fisher-Pearson standardized moment coefficient G1).

    Docstring source:
    https://tsfresh.readthedocs.io/en/latest/api/tsfresh.feature_extraction.html#tsfresh.feature_extraction.feature_calculators.skewness
    """
    name = "skewness"
    input_types = [Numeric]
    return_type = Numeric
    stack_on_self = False

    def get_function(self):
        return skewness



from featuretools.primitives import AggregationPrimitive
from featuretools.variable_types import Numeric
from tsfresh.feature_extraction.feature_calculators import \
    ratio_value_number_to_time_series_length


class RatioValueNumberToTimeSeriesLength(AggregationPrimitive):
    """Returns a factor which is 1 if all values in the time series occur only
    once, and below one if this is not the case. In principle, it just returns

        # unique values / # values

    Docstring source:
    https://tsfresh.readthedocs.io/en/latest/api/tsfresh.feature_extraction.html#tsfresh.feature_extraction.feature_calculators.ratio_value_number_to_time_series_length
    """
    name = "ratio_value_number_to_time_series_length"
    input_types = [Numeric]
    return_type = Numeric
    stack_on_self = False

    def get_function(self):
        return ratio_value_number_to_time_series_length



from featuretools.primitives import AggregationPrimitive
from featuretools.variable_types import Numeric
from tsfresh.feature_extraction.feature_calculators import \
    percentage_of_reoccurring_values_to_all_values


class PercentageOfReoccurringValuesToAllValues(AggregationPrimitive):
    """Returns the ratio of unique values, that are present in the time series
    more than once.

        # of data points occurring more than once / # of all data points

    This means the ratio is normalized to the number of data points in the time
    series, in contrast to the
    percentage_of_reoccurring_datapoints_to_all_datapoints.

    Docstring source:
    https://tsfresh.readthedocs.io/en/latest/api/tsfresh.feature_extraction.html#tsfresh.feature_extraction.feature_calculators.percentage_of_reoccurring_values_to_all_values
    """
    name = "percentage_of_reoccurring_values_to_all_values"
    input_types = [Numeric]
    return_type = Numeric
    stack_on_self = False

    def get_function(self):
        return percentage_of_reoccurring_values_to_all_values



from featuretools.primitives import AggregationPrimitive
from featuretools.variable_types import Numeric
from tsfresh.feature_extraction.feature_calculators import mean_abs_change


class MeanAbsChange(AggregationPrimitive):
    """Returns the mean over the absolute differences between subsequent time
    series values which is

    .. math::

        \\frac{1}{n} \\sum_{i=1,\\ldots, n-1} | x_{i+1} - x_{i}|

    Docstring source:
    https://tsfresh.readthedocs.io/en/latest/api/tsfresh.feature_extraction.html#tsfresh.feature_extraction.feature_calculators.mean_abs_change
    """
    name = "mean_abs_change"
    input_types = [Numeric]
    return_type = Numeric
    stack_on_self = False

    def get_function(self):
        return mean_abs_change



from featuretools.primitives import AggregationPrimitive
from featuretools.variable_types import Numeric
from tsfresh.feature_extraction.feature_calculators import fft_coefficient


class FftCoefficient(AggregationPrimitive):
    """Calculates the fourier coefficients of the one-dimensional discrete
    Fourier Transform for real input by fast fourier transformation algorithm

    .. math::
        A_k =  \\sum_{m=0}^{n-1} a_m \\exp \\left \\{ -2 \\pi i \\frac{m k}{n} \\right \\}, \\qquad k = 0,
        \\ldots , n-1.

    The resulting coefficients will be complex, this feature calculator can
    return the real part (attr=="real"), the imaginary part (attr=="imag), the
    absolute value (attr=""abs) and the angle in degrees (attr=="angle).

    Args:
        coeff (int) : The coefficient to return. Must be greater than or equal to zero.
        attr (str) : Controls which attribute is returned.
            Possible values are: ["real", "imag", "abs", "angle"]

    Docstring source:
    https://tsfresh.readthedocs.io/en/latest/api/tsfresh.feature_extraction.html#tsfresh.feature_extraction.feature_calculators.fft_coefficient
    """
    name = "fft_coefficient"
    input_types = [Numeric]
    return_type = Numeric
    stack_on_self = False

    def __init__(self, coeff, attr):
        self.coeff = coeff
        self.attr = attr

    def get_function(self):
        def function(x):
            param = [{'coeff': self.coeff, 'attr': self.attr}]
            return list(fft_coefficient(x, param=param))[0][1]

        return function



from featuretools.primitives import AggregationPrimitive
from featuretools.variable_types import Numeric
from tsfresh.feature_extraction.feature_calculators import \
    augmented_dickey_fuller


class AugmentedDickeyFuller(AggregationPrimitive):
    """The Augmented Dickey-Fuller test is a hypothesis test which checks
    whether a unit root is present in a time series sample.

    Args:
        attr (str) : Controls which attribute is returned. Possible values are:
            ['pvalue', 'teststat', 'usedlag'].

    Docstring source:
    https://tsfresh.readthedocs.io/en/latest/api/tsfresh.feature_extraction.html#tsfresh.feature_extraction.feature_calculators.augmented_dickey_fuller
    """
    name = "augmented_dickey_fuller"
    input_types = [Numeric]
    return_type = Numeric
    stack_on_self = False

    def __init__(self, attr):
        self.attr = attr

    def get_function(self):
        def function(x):
            param = [{'attr': self.attr}]
            return augmented_dickey_fuller(x, param=param)[0][1]

        return function



from featuretools.primitives import AggregationPrimitive
from featuretools.variable_types import Numeric
from tsfresh.feature_extraction.feature_calculators import value_count


class ValueCount(AggregationPrimitive):
    """Count occurrences of `value` in time series x.

    Args:
        value (float) : The value to be counted.

    Docstring source:
    https://tsfresh.readthedocs.io/en/latest/api/tsfresh.feature_extraction.html#tsfresh.feature_extraction.feature_calculators.value_count
    """
    name = "value_count"
    input_types = [Numeric]
    return_type = Numeric
    stack_on_self = False

    def __init__(self, value):
        self.value = value

    def get_function(self):
        def function(x):
            return value_count(x, value=self.value)

        return function



from featuretools.primitives import AggregationPrimitive
from featuretools.variable_types import Numeric
from tsfresh.feature_extraction.feature_calculators import cid_ce


class CidCe(AggregationPrimitive):
    """This function calculator is an estimate for a time series complexity [1]
    (A more complex time series has more peaks, valleys etc.). It calculates
    the value of

    .. math::

        \\sqrt{ \\sum_{i=0}^{n-2lag} ( x_{i} - x_{i+1})^2 }

    .. rubric:: References

    |  [1] Batista, Gustavo EAPA, et al (2014).
    |  CID: an efficient complexity-invariant distance for time series.
    |  Data Mining and Knowledge Discovery 28.3 (2014): 634-669.

    Args:
        normalize (bool) : Should the time series be z-transformed?

    Docstring source:
    https://tsfresh.readthedocs.io/en/latest/api/tsfresh.feature_extraction.html#tsfresh.feature_extraction.feature_calculators.cid_ce
    """
    name = "cid_ce"
    input_types = [Numeric]
    return_type = Numeric
    stack_on_self = False

    def __init__(self, normalize):
        self.normalize = normalize

    def get_function(self):
        def function(x):
            return cid_ce(x, normalize=self.normalize)

        return function



from featuretools.primitives import AggregationPrimitive
from featuretools.variable_types import Numeric
from tsfresh.feature_extraction.feature_calculators import minimum


class Minimum(AggregationPrimitive):
    """Calculates the lowest value of the time series x.

    Docstring source:
    https://tsfresh.readthedocs.io/en/latest/api/tsfresh.feature_extraction.html#tsfresh.feature_extraction.feature_calculators.minimum
    """
    name = "minimum"
    input_types = [Numeric]
    return_type = Numeric
    stack_on_self = False

    def get_function(self):
        return minimum



from setuptools import find_packages, setup

setup(
    name='featuretools_tsfresh_primitives',
    version='0.3.0',
    author='Feature Labs, Inc.',
    author_email='support@featurelabs.com',
    description='TSFresh primitives for featuretools',
    license='MIT',
    install_requires=open('requirements.txt').readlines(),
    tests_require=open('test-requirements.txt').readlines(),
    long_description=open('README.md').read(),
    long_description_content_type='text/markdown',
    packages=find_packages(),
    include_package_data=True,
    entry_points={
        'featuretools_plugin': [
            'tsfresh = featuretools_tsfresh_primitives.__init__',
        ],
    },
)



from featuretools.primitives import AggregationPrimitive
from featuretools.variable_types import Boolean, Numeric
from tsfresh.feature_extraction.feature_calculators import \
    variance_larger_than_standard_deviation


class VarianceLargerThanStandardDeviation(AggregationPrimitive):
    """Boolean variable denoting if the variance of x is greater than its
    standard deviation. Is equal to variance of x being larger than 1

    Docstring source:
    https://tsfresh.readthedocs.io/en/latest/api/tsfresh.feature_extraction.html#tsfresh.feature_extraction.feature_calculators.variance_larger_than_standard_deviation
    """
    name = "variance_larger_than_standard_deviation"
    input_types = [Numeric]
    return_type = Boolean
    stack_on_self = False

    def get_function(self):
        return variance_larger_than_standard_deviation



from featuretools.primitives import AggregationPrimitive
from featuretools.variable_types import Numeric
from tsfresh.feature_extraction.feature_calculators import \
    energy_ratio_by_chunks


class EnergyRatioByChunks(AggregationPrimitive):
    """Calculates the sum of squares of chunk i out of N chunks expressed as a
    ratio with the sum of squares over the whole series.

    Args:
        num_segments (int) : Number of segments to divide the series into.
        segment_focus (int) : Segment number (starting at zero) to return a feature on.

    Docstring source:
    https://tsfresh.readthedocs.io/en/latest/api/tsfresh.feature_extraction.html#tsfresh.feature_extraction.feature_calculators.energy_ratio_by_chunks
    """
    name = "energy_ratio_by_chunks"
    input_types = [Numeric]
    return_type = Numeric
    stack_on_self = False

    def __init__(self, num_segments, segment_focus):
        self.num_segments = num_segments
        self.segment_focus = segment_focus

    def get_function(self):
        def function(x):
            param = [{'num_segments': self.num_segments, 'segment_focus': self.segment_focus}]
            return energy_ratio_by_chunks(x, param)[0][1]

        return function



from featuretools.primitives import AggregationPrimitive
from featuretools.variable_types import Numeric
from tsfresh.feature_extraction.feature_calculators import number_peaks


class NumberPeaks(AggregationPrimitive):
    """Calculates the number of peaks of at least support n in the time series
    x. A peak of support n is defined as a subsequence of x where a value
    occurs, which is bigger than its n neighbours to the left and to the right.

    Hence in the sequence

    `x = [3, 0, 0, 4, 0, 0, 13]`

    4 is a peak of support 1 and 2 because in the subsequences

    `[0, 4, 0]`
    `[0, 0, 4, 0, 0]`

    4 is still the highest value. Here, 4 is not a peak of support 3 because 13
    is the 3th neighbour to the right of 4 and its bigger than 4.

    Args:
        n (int) : The support of the peak.

    Docstring source:
    https://tsfresh.readthedocs.io/en/latest/api/tsfresh.feature_extraction.html#tsfresh.feature_extraction.feature_calculators.number_peaks
    """
    name = "number_peaks"
    input_types = [Numeric]
    return_type = Numeric
    stack_on_self = False

    def __init__(self, n):
        self.n = n

    def get_function(self):
        def function(x):
            return number_peaks(x, n=self.n)

        return function



from featuretools.primitives import AggregationPrimitive
from featuretools.variable_types import Numeric
from tsfresh.feature_extraction.feature_calculators import number_cwt_peaks


class NumberCwtPeaks(AggregationPrimitive):
    """This feature calculator searches for different peaks in x. To do so, x
    is smoothed by a ricker wavelet and for widths ranging from 1 to n. This
    feature calculator returns the number of peaks that occur at enough width
    scales and with sufficiently high Signal-to-Noise-Ratio (SNR).

    Args:
        n (int) : Maximum width to consider.

    Docstring source:
    https://tsfresh.readthedocs.io/en/latest/api/tsfresh.feature_extraction.html#tsfresh.feature_extraction.feature_calculators.number_cwt_peaks
    """
    name = "number_cwt_peaks"
    input_types = [Numeric]
    return_type = Numeric
    stack_on_self = False

    def __init__(self, n):
        self.n = n

    def get_function(self):
        def function(x):
            return number_cwt_peaks(x, n=self.n)

        return function
